LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

ENTITY sram_ctrl IS
	PORT (
		clk, reset : IN std_logic;
		mem : IN std_logic;
		rw : IN std_logic;
		addr : IN std_logic_vector(17 DOWNTO 0);
		data_f2s : IN std_logic_vector(7 DOWNTO 0);
		ready : OUT std_logic;
		data_s2f_r : OUT std_logic_vector(7 DOWNTO 0);
		--data_s2f_ur : OUT std_logic_vector(7 DOWNTO 0);
		ad : OUT std_logic_vector(17 DOWNTO 0);
		we_n, oe_n : OUT std_logic;
		dio : INOUT std_logic_vector(7 DOWNTO 0);
		ce_n : OUT std_logic;
		SRAM_UB_N   : out std_logic; -- upper byte mask
		SRAM_LB_N   : out std_logic; -- lower byte mask	
		LED1, LED2 : OUT std_logic		
	);
END sram_ctrl;

ARCHITECTURE arch OF sram_ctrl IS
	TYPE state_type IS (idle, r1, r2, w1, w2);
	SIGNAL state_reg, state_next : state_type;
	SIGNAL data_f2s_reg, data_f2s_next : 
	std_logic_vector(7 DOWNTO 0);
	SIGNAL data_s2f_reg, data_s2f_next : 
	std_logic_vector(7 DOWNTO 0);
	SIGNAL addr_reg, addr_next : 
	std_logic_vector(17 DOWNTO 0);
	SIGNAL we_buf, oe_buf, tri_buf : std_logic;
	SIGNAL we_reg, oe_reg, tri_reg : std_logic;

BEGIN
	PROCESS (clk, reset)
	BEGIN
		IF (reset = '1') THEN
			state_reg <= idle;
			addr_reg <= (OTHERS => '0');
			data_f2s_reg <= (OTHERS => '0');
			data_s2f_reg <= (OTHERS => '0');
			tri_reg <= '1';
			we_reg <= '1';
			oe_reg <= '1';
			SRAM_LB_N <= '1'; -- mask low byte
			SRAM_UB_N <= '1'; -- mask high byte
			LED1 <= '0';
			LED2 <= '0';
		ELSIF (clk'event AND clk = '1') THEN
			state_reg <= state_next;
			addr_reg <= addr_next;
			data_f2s_reg <= data_f2s_next;
			data_s2f_reg <= data_s2f_next;
			tri_reg <= tri_buf;
			we_reg <= we_buf;
			oe_reg <= oe_buf;
			SRAM_LB_N <= '0'; -- unmask low byte
			SRAM_UB_N <= '0'; -- unmask high byte
		END IF;
	END PROCESS;
	
	-- next state logic
	PROCESS (state_reg, mem, rw, dio, addr, data_f2s, 
	data_f2s_reg, data_s2f_reg, addr_reg)
		BEGIN
			addr_next <= addr_reg;
			data_f2s_next <= data_f2s_reg;
			data_s2f_next <= data_s2f_reg;
			ready <= '0';
			CASE state_reg IS
				WHEN idle => 
					IF (mem = '0') THEN
						state_next <= idle;
					ELSE
						addr_next <= addr;
						IF (rw = '0') THEN -- write
							state_next <= w1;
							data_f2s_next <= data_f2s;
						ELSE
							state_next <= r1;
						END IF;
					END IF;
					ready <= '1';
				WHEN w1 => 
					state_next <= w2;
				WHEN w2 => 
					state_next <= idle;
				WHEN r1 => 
					state_next <= r2;
				WHEN r2 => 
					data_s2f_next <= dio;
					state_next <= idle;
			END CASE;
		END PROCESS;
		-- look-ahead output logic
		PROCESS (state_next)
			BEGIN
				tri_buf <= '1';
				we_buf <= '1';
				oe_buf <= '1';
				CASE state_next IS
					WHEN idle => 
					WHEN w1 => 
						tri_buf <= '0';
						we_buf <= '0';			
					WHEN w2 => 
						tri_buf <= '0';
					WHEN r1 => 
						oe_buf <= '0';
					WHEN r2 => 
						oe_buf <= '0';
				END CASE;
			END PROCESS;
			
			-- to main system
			data_s2f_r <= data_s2f_reg;
			--data_s2f_ur <= dio;
			-- to SRAM
			we_n <= we_reg;
			oe_n <= oe_reg;
			ad <= addr_reg;
			-- I/O for SRAM chip
			ce_n <= '0';
			dio <= data_f2s_reg WHEN tri_reg = '0'
			       ELSE (OTHERS => 'Z');
END arch;